@@ -15,14 +15,9 @@ package source
 
 import (
 	"path/filepath"
+	"strings"
 	"testing"
 
-	"github.com/gohugoio/hugo/helpers"
-
-	"github.com/spf13/viper"
-
-	"github.com/gohugoio/hugo/hugofs"
-	"github.com/spf13/afero"
 	"github.com/stretchr/testify/require"
 )
 
@@ -57,44 +52,10 @@ func TestFileInfo(t *testing.T) {
 
 		}},
 	} {
-		f := s.NewFileInfo(this.base, this.filename, false, nil)
+		path := strings.TrimPrefix(this.filename, this.base)
+		f, err := s.NewFileInfoFrom(path, this.filename)
+		assert.NoError(err)
 		this.assert(f)
 	}
 
 }
-
-func TestFileInfoLanguage(t *testing.T) {
-	assert := require.New(t)
-	langs := map[string]bool{
-		"sv": true,
-		"en": true,
-	}
-
-	m := afero.NewMemMapFs()
-	lfs := hugofs.NewLanguageFs("sv", langs, m)
-	v := viper.New()
-	v.Set("contentDir", "content")
-
-	fs := hugofs.NewFrom(m, v)
-
-	ps, err := helpers.NewPathSpec(fs, v)
-	assert.NoError(err)
-	s := SourceSpec{Fs: lfs, PathSpec: ps}
-	s.Languages = map[string]interface{}{
-		"en": true,
-	}
-
-	err = afero.WriteFile(lfs, "page.md", []byte("abc"), 0777)
-	assert.NoError(err)
-	err = afero.WriteFile(lfs, "page.en.md", []byte("abc"), 0777)
-	assert.NoError(err)
-
-	sv, _ := lfs.Stat("page.md")
-	en, _ := lfs.Stat("page.en.md")
-
-	fiSv := s.NewFileInfo("", "page.md", false, sv)
-	fiEn := s.NewFileInfo("", "page.en.md", false, en)
-
-	assert.Equal("sv", fiSv.Lang())
-	assert.Equal("en", fiEn.Lang())
-}
